/*
 * Your installation or use of this SugarCRM file is subject to the applicable
 * terms available at
 * http://support.sugarcrm.com/06_Customer_Center/10_Master_Subscription_Agreements/.
 * If you do not agree to all of the applicable terms or do not have the
 * authority to bind the entity as an authorized representative, then do not
 * install or use this SugarCRM file.
 *
 * Copyright (C) SugarCRM Inc. All rights reserved.
 */
/**
 * @class View.Views.Base.CustomerJourneyDashletView
 * @alias SUGAR.App.view.views.BaseCustomerJourneyDashletView
 * @extends View.View
 */
({
    plugins: ['Dashlet'],

    initialize: function (options) {
        this._super('initialize', [options]);
        this.on("render", _.bind(this.buildList, this))
        //this.model.on("change", _.bind(this.loadData, this));
    },


    initDashlet: function () {
        var API_VERSION = 'v10';
        var tokensData = {"module": this.module, 'id': this.model.get('id')};

        var requests = [
            {
                "url": API_VERSION + "/pandadoctokens", "method": "GET", "data": JSON.stringify(tokensData)
            },
        ];

        switch (this.module) {
            case "Opportunities":
                requests.push(
                    {
                        "url": API_VERSION + "/" + this.module + "/" + this.model.get('id') + "/link/contacts?max_num=-1", "method": "GET"
                    },
                    {
                        "url": API_VERSION + "/" + this.module + "/" + this.model.get('id') + "/link/revenuelineitems?max_num=-1",
                        "method": "GET"
                    },
                    {
                      "url": API_VERSION + "/" + this.module + "/" + this.model.get('id') + "/link/products?max_num=-1",
                      "method": "GET"
                    },
                    {
                      "url": API_VERSION + "/pandadocconfig",
                      "method": "GET"
                    }
                );
                break;
            case "Accounts":
                requests.push({
                    "url": API_VERSION + "/" + this.module + "/" + this.model.get('id') + "/link/contacts?max_num=-1", "method": "GET"
                });
                break;
            case"Contacts":
                requests.push({
                    "url": API_VERSION + "/" + this.module + "/" + this.model.get('id'), "method": "GET"
                });
            case "Quotes":
                requests.push({
                    "url": API_VERSION + "/" + this.module + "/" + this.model.get('id') + "/link/products?max_num=-1",
                    "method": "GET"
                });
                break;
            default:
                break;
        }

        this.requests = requests;
    },

    buildProductsList: function (RLIs) {
        var self = this;
        self.products = [];
        if (!_.isEmpty(RLIs.records)) {
            _.each(RLIs.records, function (rli) {
                var product = {
                    sku: rli.id,
                    name: rli.name || rli.product_template_name || "",
                    price: rli.discount_price || 0,
                    qty: rli.quantity || 0,
                    description: rli.description || "",
                    currency: rli.currency_name || "USD",
                    custom_fields: _.omit(rli, function(v) {
                        return _.isUndefined(v) || _.isNull(v);
                    })
                };
                self.products.push(product);
            })
        }
    },

    buildProductsListFromQLI: function (QLIs) {
        var self = this;
        self.products = [];
        if (!_.isEmpty(QLIs.records)) {
            _.each(QLIs.records, function (qli) {
                var price = 0;
                if (qli.quantity > 0) {
                    price = qli.total_amount / qli.quantity;
                }
                var product = {
                    sku: qli.id,
                    name: qli.name || qli.product_template_name || "",
                    price: price,
                    qty: qli.quantity || 0,
                    description: qli.description || "",
                    currency: qli.currency_name || "USD",
                    custom_fields: _.omit(qli, function(v) {
                        return _.isUndefined(v) || _.isNull(v);
                    })
                };
                self.products.push(product);
            })
        }
    },

    buildTokensList: function (tokens) {
        this.tokens = {
            "tokens": tokens || {}
        }
    },

    buildList: function () {
        var self = this;
        var doclist = new PandaDoc.DocList({mode: PandaDoc.DOC_LIST_MODE.LIST});
        doclist.init({
            el: "#documents_list",
            initOptions: {
                isExtension: true,
                externalEditor: true,
                showTokens: true,
                provider: {
                    name: 'sugar-crm',
                }
            },
            data: {
                metadata: {
                    'model_id': self.model.get("id")
                }
            },
            events: {
                'onInit': function () {

                },
                'onDocumentCreate': function () {
                    self.openEditor();
                },
                'onDocumentOpen': function (data) {
                    if(data && data.documentUrl) {
                        var _win = window.open(data.documentUrl, '_blank');
                        _win.focus();
                    }
                },
                onTokensOpen: function () {
                    var get_args = {
                        module: self.module,
                        id: self.model.get("id")
                    }
                    app.api.call('read', app.api.buildURL('pandadoctokens', null, null, get_args), null, {
                        success: function (data) {
                            self.tokens = {
                                'tokens': data,
                                'customHelpText': 'There is no data on this page to be merged into PandaDoc. ' +
                                    'Please fill in the default fields or add custom ones to use them as tokens. ' +
                                    'This can be done by going to SugarCRM settings > Admin > PandaDoc Tokens.'
                            };
                            doclist.showAvailableTokens(self.tokens);
                        }
                    })
                }
            }
        });
        this.doclist = doclist;
    },

    openEditor: function () {
        var self = this;

        app.api.call('create', app.api.buildURL(null, 'bulk'), {requests: self.requests}, {
            success: function (data) {
                var tokens = data[0].contents || {};
                var contacts = {};

                switch (self.module) {
                    case "Opportunities":
                        contacts = data[1].contents || {};
                        var RLIs = data[2].contents || {};
                        var QLIs = data[3].contents || {};
                        var pandadocConfig = data[4].contents || {};

                        self.buildRecipientsList(contacts);
                        if (pandadocConfig.pandadoc_inject == 'qli') {
                            self.buildProductsListFromQLI(QLIs);
                        } else {
                            self.buildProductsList(RLIs);
                        }
                        break;
                    case "Accounts":
                        contacts = data[1].contents || {};
                        self.buildRecipientsList(contacts);
                        break;
                    case "Contacts":
                        var records = data[1].contents ? [data[1].contents] : [],
                            contacts = {'records': records};

                        self.buildRecipientsList(contacts);
                        break;
                    case "Quotes":
                        var QLIs = data[1].contents || {};
                        self.buildProductsListFromQLI(QLIs);
                        break;
                    default:
                        break;
                }
                self.buildTokensList(tokens);
                self.showDocumentEditor();
            }
        })
    },

    buildRecipientsList: function (contacts) {
        var self = this;
        self.recipients = [];

        if (!_.isEmpty(contacts.records)) {
            _.each(contacts.records, function (contact) {

                var recipient = {
                    first_name: contact.first_name || "",
                    last_name: contact.last_name || "",
                    email: contact.email1 || "",
                    phone: contact.phone_mobile || contact.phone_work || contact.phone_home || contact.phone_other || "",
                    company: contact.account_name || "",
                    roleName: contact.opportunity_role || ""
                };
                self.recipients.push(recipient);
            })
        }
    },

    showDocumentEditor: function () {
        var self = this;
        var editor = new PandaDoc.DocEditor();
        editor.showAsModal({
            el: 'body',
            data: {
                docName: self.model.get("name"),
                metadata: { //that's how you link newly create document with your custom data
                    'sugarcrm.model_id': self.model.get('id'),
                    'model_id': self.model.get("id")
                },
                //populate tokens and widgets in a template
                widgets: _.clone(self.tokens.tokens),
                tokens: self.tokens.tokens,
                recipients: self.recipients,
                //pricing tables data
                items: self.products,
            },
            cssClass: 'document_editor',
            initOptions: {
                isExtension: true,
                provider: {
                    name: 'sugar-crm',
                }
            },
            events: {
                'onInit': function () {

                },
                'onDocumentCreated': function () {

                },
                'onDocumentSent': function () {

                },
                'onClose': function () {
                    // Refreshing Documents List when closing
                    // editor window. Need to do this to refresh it.
                    if (this.doc) {
                        self.doclist.destroy();
                    }
                    self.render();
                }
            }
        });
    }
})
