<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*
 * Your installation or use of this SugarCRM file is subject to the applicable
 * terms available at
 * http://support.sugarcrm.com/06_Customer_Center/10_Master_Subscription_Agreements/.
 * If you do not agree to all of the applicable terms or do not have the
 * authority to bind the entity as an authorized representative, then do not
 * install or use this SugarCRM file.
 *
 * Copyright (C) SugarCRM Inc. All rights reserved.
 */
/*********************************************************************************

 * Description:  TODO: To be written.
 * Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
 * All Rights Reserved.
 * Contributor(s): ______________________________________..
 ********************************************************************************/

class ViewPandadoc extends SugarView
{

    /**
     * List of modules that does not support dashlets or there is no sense to add it.
     *
     * @var array
     */
    private $unsupportedModules = array (
        'Bugs',
        'Calendar',
        'Campaigns',
        'Contracts',
        'Documents',
        'Emails',
        'Forecasts',
        'Home',
        'Notes',
        'ProspectLists',
        'Tags',
        'pmse_Business_Rules',
        'pmse_Emails_Templates',
        'pmse_Inbox',
        'pmse_Project',
        'Project',
        'Prospects',
        'Reports',
    );

    public $defaultAllowedModules = array(
        'Opportunities',
        'Accounts',
        'Contacts',
        'Users'
    );

    public $allowedModules = array();


    /**
     * @see SugarView::display()
     */
    public function display()
    {
        global $mod_strings, $moduleList;

        $this->allowedModules = array_unique(array_merge($this->defaultAllowedModules, $this->loadAllowedModules()));
        sort($this->allowedModules);

        $fields = array();
        foreach($this->allowedModules as $module){
            $fields[$module] = $this->loadFieldsForModule($module);
        }

        $admin = new Administration();
        $pandaDocTokensSettings = $admin->retrieveSettings('pandadoc');
        $pandaDocTokens = isset($pandaDocTokensSettings->settings['pandadoc_tokens']) ?
            $pandaDocTokensSettings->settings['pandadoc_tokens'] :
            '{}';

        /*
         * Fix for backward compatibility Sugar ver < 7.11.0
         * @see Administration::retrieveSettings()
        */
        $pandaDocTokens = is_string($pandaDocTokens) ? json_decode(html_entity_decode($pandaDocTokens), true) : $pandaDocTokens;

        $allModules = $moduleList;
        $supportedModules = array_diff($allModules, $this->getUnsupportedModules());
        sort($supportedModules);

        $this->ss->register_function('sugar_translate', array($this, 'sugarTranslate'));

        $inject = isset($pandaDocTokensSettings->settings['pandadoc_inject']) ? $pandaDocTokensSettings->settings['pandadoc_inject'] : 'rli';
        $this->ss->assign('INJECT', $inject);
        $this->ss->assign("MODULES", $supportedModules);
        $this->ss->assign("DEFAULT_MODULES", $this->defaultAllowedModules);
        $this->ss->assign("ALLOWED_MODULES", $this->allowedModules);
        $this->ss->assign("MOD_FIELDS", $fields);
        $this->ss->assign("TOKENS", $pandaDocTokens);
        $content = $this->ss->fetch('custom/modules/Administration/tpls/pandadoc.tpl');

        $this->ss->assign("PANDADOC_TITLE", $mod_strings['LBL_PANDADOC_TITLE']);
        $this->ss->assign("CONTENT",$content);
        $this->ss->display('custom/modules/Administration/tpls/wrapper.tpl');
    }

    /**
     * Translates internal module name to more readable variant.
     *
     * @param $params array Translating params.
     * @param $smarty Smarty Smarty object.
     * @return string Translated string.
     */
    public function sugarTranslate($params, &$smarty)
    {
        $underscored = isset($params['underscored']) &&  $params['underscored'] === true;
        $value = $params['value'];

        $translated = translate($value);
        return $underscored ? str_replace(' ', '_', $translated) : $translated;
    }

    protected function loadFieldsForModule($module){
        $bean = BeanFactory::getBean($module);
        $fields = $bean->getFieldDefinitions();
        $beanFields = $this->filterFields($fields);
        return $beanFields;
    }

    protected function filterFields($fields){
        $fieldsArray = array();
        foreach($fields as $name=>$def){
             if($def['type'] == 'id' ||
                 (isset($def['dbType']) && $def['dbType'] == "id") ||
                 $def['type'] == 'link' ||
                 $def['type'] == 'tag'
             ) {
                 unset($fields[$name]);
             }
            else {
                $fieldsArray[]= $name;
            }
        }

        return $fieldsArray;
    }

    /**
     * Returns array of allowed modules.
     *
     * @return array
     */
    private function loadAllowedModules()
    {
        $admin = new Administration();
        $pandaDocSettings = $admin->retrieveSettings('pandadoc');
        $pdAllowedModules = array_key_exists('pandadoc_allowedModules', $pandaDocSettings->settings)
            ? $pandaDocSettings->settings['pandadoc_allowedModules'] : array();

        return is_string($pdAllowedModules) ? json_decode(html_entity_decode($pdAllowedModules), true) : $pdAllowedModules;
    }

    /**
     * Returns list of unsupported modules.
     * Calculated based on Sugar version.
     *
     * @return array
     */
    private function getUnsupportedModules()
    {
        global $sugar_version;

        $unsupportedModules = $this->unsupportedModules;
        if (version_compare($sugar_version, '7.9.0') < 0) {
            $unsupportedModules[] = 'Quotes';
        }
        return $unsupportedModules;
    }
}
